"""Image Validation module."""


def allowed_base64_image(image: str) -> bool:
    """Check if base64 image has an allowed format.
    :param image:
    :return:
    """
    if not image.startswith('data:image/'):
        return False
    return image[11:14] in {'png', 'jpg', 'jpeg', 'gif'}



""" Objects operations file """
import datetime


def convert_to_string(input_value):
    """
    :param input_value:
    :return:
    """
    return input_value.__str__()


def datetime2str(value: datetime.datetime):
    """
    :param value:
        datetie instance object.
    :return:
        str with the conversion
    """
    if not isinstance(value, datetime.datetime):
        raise TypeError("Value is not datetime.")

    return value.__str__()



"""API result module"""


class ApiResult:
    """This class stores data from API operations."""
    def __init__(self, status: str = None, message_class: str = None, message: str = None):
        """Class initializer.
        :param status:
            Status.
        :param message_class:
            Message class (for UI purposes).
        :param message:
            Result message.
        """
        self.status = status
        self.message_class = message_class
        self.message = message



""" Base model automatically generated from swagger.
"""
import json
import pprint
import typing
from datetime import datetime
import six
from yk_utils.models.deserialization import deserialize_dbmodel, deserialize_model
from yk_utils.objects.object_operations import datetime2str, convert_to_string

GenericType = typing.TypeVar('T')


class Model:
    """
    Base model for swagger generated objects.
    """
    # swaggerTypes: The key is attribute name and the
    # value is attribute type.
    swagger_types = {}

    # swaggerTypesDB: The key is attribute name and the
    # value is attribute type.
    swagger_types_db = None

    # swaggerTypes: The key is attribute name and the
    # value is attribute type.
    fields_to_not_save = []

    # attributeMap: The key is attribute name and the
    # value is json key in definition.
    attribute_map = {}

    @classmethod
    def from_dict(cls: typing.Type[GenericType], dikt) -> GenericType:
        """Returns the dict as a model"""
        return deserialize_model(dikt, cls)

    @classmethod
    def from_db(cls, mongo_object) -> GenericType:
        """Returns the mongodb object as a model"""
        return deserialize_dbmodel(json.loads(
            json.dumps(mongo_object, default=convert_to_string)), cls)

    def to_db(self):
        """Returns the model properties as a dict

        :rtype: dict
        """
        result = {}
        if self.swagger_types_db is None:
            self.swagger_types_db = self.swagger_types

        for attr, _ in six.iteritems(self.swagger_types_db):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_db() if hasattr(x, "to_db") else x,
                    value
                ))
            elif hasattr(value, "to_db"):
                result[attr] = value.to_db()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_db())
                    if hasattr(item[1], "to_db") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_dict(self):
        """Returns the model properties as a dict

        :rtype: dict
        """
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            attr_name = self.attribute_map[attr] if self.attribute_map else attr
            if isinstance(value, list):
                result[attr_name] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_enumdict"):
                result[attr_name] = value.to_enumdict()
            elif hasattr(value, "to_dict"):
                result[attr_name] = value.to_dict()
            elif isinstance(value, datetime):
                result[attr_name] = datetime2str(value)
            elif isinstance(value, dict):
                result[attr_name] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif value is not None:
                result[attr_name] = value

        return result

    def to_str(self):
        """Returns the string representation of the model

        :rtype: str
        """
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""Error model
"""
from __future__ import absolute_import

from .deserialization import deserialize_model
from .base_model import Model


ErrorSchema = {
    "type": "object",
    "title": "The Error Schema",
    "properties": {
        "message": {
            "type": "string",
            "title": "Error message",
            "pattern": "^(.*)$"
        }
    }
}


class Error(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, message: str = None):  # noqa: E501
        """Document - a model defined in Swagger

        :param message: The message of this Error.  # noqa: E501
        :type type: str
        """
        self.swagger_types = {
            'message': str
        }

        self.attribute_map = {
            'message': 'message'
        }

        self._message = message

    @classmethod
    def from_dict(cls, dikt) -> 'Error':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Error object.  # noqa: E501
        :rtype: Error
        """
        return deserialize_model(dikt, cls)

    @property
    def message(self) -> str:
        """Gets the message of this Error.


        :return: The message of this Error.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message: str):
        """Sets the message of this Error.


        :param type: The message of this Error.
        :type message: str
        """
        self._message = message



""" File handling """
import json


def read_json_from_file(filename: str, encoding: str = 'utf-8') -> dict:
    """Load json configuration file.
    :param filename:
        Path to file.
    :param encoding:
        Encoding in which the file is read.
    :return:
        Loaded configuration as a dict.
    """
    if not filename:
        raise ValueError('Filename must be provided.')

    config = None
    with open(filename, encoding=encoding) as file:
        config = json.load(file)
    return config


def write_json_to_file(dictionary: dict, filename: str, encoding: str = 'utf-8') -> None:
    """
    Write dictionary to file
    :param dictionary:
        dictionary to be stored.
    :param filename:
        path to config file.
    :param encoding:
        Encoding in which the file is read
    """
    if not filename:
        raise ValueError('Filename must be provided.')

    with open(filename, 'w', encoding=encoding) as outfile:
        json.dump(dictionary, outfile, indent=4)



"""Face Authentication module"""
import json
import requests
from bs4 import BeautifulSoup
from yk_utils.apis import ApiResult
from yk_utils.images import allowed_base64_image


class FaceAuthentication:
    """This class is a wrapper for the YooniK Web Product, an API for face authentication.
    """
    def __init__(self, api_url: str, api_key: str):
        """Class initializer
        :param api_url:
            YooniK Authentication API URL.
        :param api_key:
            YooniK Authentication API Key.
        """
        self.api_url = api_url
        self.api_key = api_key

    @staticmethod
    def parse_response_error(html_text: str, extra_response_codes: dict = None) -> str:
        """Parse HTML error response
        :param html_text:
            HTML error message.
        :param extra_response_codes:
            Dictionary with additional response codes and description pairs, to search on html_text.
        :return:
            Parsed error message.
        """
        html = BeautifulSoup(markup=html_text, features="html.parser")
        message = html.text
        if html.p:
            inner_html = BeautifulSoup(markup=html.p.text, features="html.parser")
            message = inner_html.text if inner_html.p is None else inner_html.p.text

        if "face_not_found" in message:
            message = "Could not find a face in the image."
        elif "multiple_faces" in message:
            message = "The image has more than one person."
        elif "quality_failed" in message:
            message = "The provided image does not have enough quality."
        elif "mobile_app_no_response" in message:
            message = "Could not receive result from mobile app."
        elif extra_response_codes:
            for code in extra_response_codes:
                if code in message:
                    message = extra_response_codes[code]
                    break
        else:
            message = "An error occurred. Please contact your systems administrator."
            print(f"ERROR: {html.text}")
        return message

    @staticmethod
    def parse_response_status(status: str) -> str:
        """Create a message from the response status data
        :param status:
            Status of the operation.
        :return:
            Resulting message to be sent to the UI.
        """
        message = status
        if status == 'SUCCESS':
            message = "Face authentication successful"
        elif status == 'NEW_USER':
            message = "Face signup successful"
        elif status == 'USER_NOT_FOUND':
            message = "User not registered"
        elif status == 'FAILED':
            message = "Face authentication failed"
        return message

    def request_face_authentication(self, user_id: str, user_photo: str, user_attributes: dict = None,
                                    create_if_new: bool = True) -> ApiResult:
        """Perform a face authentication request to YooniK API.
        :param user_id:
            User ID to be authenticated.
        :param user_photo:
            User selfie image.
        :param user_attributes:
            User attributes to be saved.
        :param create_if_new:
            Create new user if the provided user ID doesn't exist.
        :return:
            Face authentication result object.
        """
        face_authentication_result = ApiResult(status='FAILED', message_class='text-danger',
                                               message='Face authentication failed')

        if allowed_base64_image(user_photo):
            yoonik_request_data = {
                'user_id': user_id,
                'user_photo': user_photo.split('base64,')[1],
                'create_if_new': create_if_new
            }
            if user_attributes:
                yoonik_request_data['user_attributes'] = user_attributes
            response = requests.post(
                self.api_url,
                headers={'x-api-key': self.api_key},
                json=yoonik_request_data
            )
            if response.ok:
                result = json.loads(response.text)
                face_authentication_result.status = result['status']
                face_authentication_result.message_class = 'text-success' if \
                    face_authentication_result.status in ('SUCCESS', 'NEW_USER') else 'text-danger'
                face_authentication_result.message = self.parse_response_status(face_authentication_result.status)
            else:
                face_authentication_result.message = f'Ups! {self.parse_response_error(response.text)}'

        return face_authentication_result

    def request_account_deletion(self, user_id: str) -> bool:
        """Delete a user from YooniK API.
        :param user_id:
            User ID to be deleted.
        :return:
            True if user successfully deleted.
        """
        response = requests.delete(
            self.api_url,
            headers={'x-api-key': self.api_key},
            json={'user_id': user_id}
        )
        return response.ok



"""YoonikApiException module
"""


class YoonikApiException(Exception):
    """Custom Exception for the YooniK APIs."""
    def __init__(self, status_code: int, message: str):
        """ Class initializer.
        :param status_code: HTTP response status code.
        :param message: Error message.
        """
        super().__init__()
        self.status_code = status_code
        self.message = message

    def __str__(self):
        return 'Error when calling YooniK API:\n' + \
            f'\tstatus_code: {self.status_code}\n' + \
            f'\tmessage: {self.message}\n'



""" APIs Module Init """
from .base_url import BaseUrl
from .key import Key
from .yoonik_api_exception import YoonikApiException
from .http_requests import request
from .api_result import ApiResult
from .face_authentication import FaceAuthentication
